@@ -60,11 +60,15 @@ struct _GStringChunk
  * g_str_equal:
  * @v1: a key
  * @v2: a key to compare with @v1
- * 
- * Compares two strings for byte-by-byte equality and returns %TRUE 
- * if they are equal. It can be passed to g_hash_table_new() as the 
+ *
+ * Compares two strings for byte-by-byte equality and returns %TRUE
+ * if they are equal. It can be passed to g_hash_table_new() as the
  * @key_equal_func parameter, when using strings as keys in a #GHashTable.
  *
+ * Note that this function is primarily meant as a hash table comparison
+ * function. For a general-purpose, %NULL-safe string comparison function,
+ * see g_strcmp0().
+ *
  * Returns: %TRUE if the two keys match
  */
 gboolean
@@ -247,22 +251,22 @@ g_string_chunk_insert (GStringChunk *chunk,
  * g_string_chunk_insert_const:
  * @chunk: a #GStringChunk
  * @string: the string to add
- * 
- * Adds a copy of @string to the #GStringChunk, unless the same 
- * string has already been added to the #GStringChunk with 
+ *
+ * Adds a copy of @string to the #GStringChunk, unless the same
+ * string has already been added to the #GStringChunk with
  * g_string_chunk_insert_const().
- * 
- * This function is useful if you need to copy a large number 
- * of strings but do not want to waste space storing duplicates. 
- * But you must remember that there may be several pointers to 
- * the same string, and so any changes made to the strings 
+ *
+ * This function is useful if you need to copy a large number
+ * of strings but do not want to waste space storing duplicates.
+ * But you must remember that there may be several pointers to
+ * the same string, and so any changes made to the strings
  * should be done very carefully.
- * 
- * Note that g_string_chunk_insert_const() will not return a 
- * pointer to a string added with g_string_chunk_insert(), even 
+ *
+ * Note that g_string_chunk_insert_const() will not return a
+ * pointer to a string added with g_string_chunk_insert(), even
  * if they do match.
- * 
- * Returns: a pointer to the new or existing copy of @string 
+ *
+ * Returns: a pointer to the new or existing copy of @string
  *          within the #GStringChunk
  */
 gchar*
@@ -291,26 +295,26 @@ g_string_chunk_insert_const (GStringChunk *chunk,
  * g_string_chunk_insert_len:
  * @chunk: a #GStringChunk
  * @string: bytes to insert
- * @len: number of bytes of @string to insert, or -1 to insert a 
- *     nul-terminated string 
- * 
- * Adds a copy of the first @len bytes of @string to the #GStringChunk. 
+ * @len: number of bytes of @string to insert, or -1 to insert a
+ *     nul-terminated string
+ *
+ * Adds a copy of the first @len bytes of @string to the #GStringChunk.
  * The copy is nul-terminated.
- * 
+ *
  * Since this function does not stop at nul bytes, it is the caller's
- * responsibility to ensure that @string has at least @len addressable 
+ * responsibility to ensure that @string has at least @len addressable
  * bytes.
  *
- * The characters in the returned string can be changed, if necessary, 
+ * The characters in the returned string can be changed, if necessary,
  * though you should not change anything after the end of the string.
- * 
+ *
  * Return value: a pointer to the copy of @string within the #GStringChunk
- * 
+ *
  * Since: 2.4
- **/
+ */
 gchar*
 g_string_chunk_insert_len (GStringChunk *chunk,
-			   const gchar  *string, 
+			   const gchar  *string,
 			   gssize        len)
 {
   gssize size;
@@ -322,7 +326,7 @@ g_string_chunk_insert_len (GStringChunk *chunk,
     size = strlen (string);
   else
     size = len;
-  
+
   if ((chunk->storage_next + size + 1) > chunk->this_size)
     {
       gsize new_size = nearest_power (chunk->default_size, size + 1);
@@ -338,9 +342,7 @@ g_string_chunk_insert_len (GStringChunk *chunk,
 
   *(pos + size) = '\0';
 
-  strncpy (pos, string, size);
-  if (len > 0)
-    size = strlen (pos);
+  memcpy (pos, string, size);
 
   chunk->storage_next += size + 1;
 
@@ -1355,7 +1357,7 @@ g_string_up (GString *string)
  * @args: the list of arguments to insert in the output
  *
  * Appends a formatted string onto the end of a #GString.
- * This function is is similar to g_string_append_printf()
+ * This function is similar to g_string_append_printf()
  * except that the arguments to the format string are passed
  * as a va_list.
  *
@@ -1452,7 +1454,7 @@ g_string_printf (GString     *string,
  * @Varargs: the parameters to insert into the format string
  *
  * Appends a formatted string onto the end of a #GString.
- * This function is is similar to g_string_sprintf() except that
+ * This function is similar to g_string_sprintf() except that
  * the text is appended to the #GString. 
  *
  * Deprecated: This function has been renamed to g_string_append_printf()
@@ -1465,7 +1467,7 @@ g_string_printf (GString     *string,
  * @Varargs: the parameters to insert into the format string
  *
  * Appends a formatted string onto the end of a #GString.
- * This function is is similar to g_string_printf() except 
+ * This function is similar to g_string_printf() except 
  * that the text is appended to the #GString.
  */
 void
